#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for `django_auto_healthchecks.healthchecks.IdempotentHealthcheckClient` class.
"""

try:
    import mock
except ImportError:
    from unittest import mock

import pytest
import django_auto_healthchecks.healthchecks as healthchecks
from . import MockSettings


class MockRequestsResponse(object):

    def __init__(self, status_code):
        self.status_code = status_code


@pytest.fixture
def healthcheck_instance():
    def mock_resolve(self):
        self._url = healthchecks.HealthcheckUrl(
            path='/path/to',
            querystring={}
        )
        self._defaultName = self._create_name()
        self.code = self.code if self.code else self._create_code()

    healthchecks.settings = MockSettings(HEALTHCHECKS={'API_KEY': 'this is a key'}, DEBUG=True)
    instance = healthchecks.Healthcheck()
    instance.resolve = lambda: mock_resolve(instance)
    return instance


@mock.patch('django_auto_healthchecks.healthchecks.requests.put')
def test_put_invokes_request(mock_put, healthcheck_instance):
    healthchecks.settings = MockSettings(HEALTHCHECKS={'API_KEY': 'this is a key'}, DEBUG=True, HOSTNAME='cronitor.io')
    healthchecks.Client.enqueue(healthcheck_instance)
    healthchecks.Client.put()
    assert mock_put.call_count == 1, "requests.put not called once"


@mock.patch('django_auto_healthchecks.healthchecks.requests.put')
def test_put_does_not_invoke_request_without_api_key(mock_put, healthcheck_instance):
    healthchecks.settings = MockSettings(HEALTHCHECKS={}, DEBUG=True, HOSTNAME='cronitor.io')
    healthchecks.Client.enqueue(healthcheck_instance)
    healthchecks.Client.put()
    assert mock_put.call_count == 0, "Unexpected call to requests.put"


@mock.patch('django_auto_healthchecks.healthchecks.requests.put', return_value=MockRequestsResponse(status_code=500))
def test_request_failure_raises_healthcheck_error(mock_put, healthcheck_instance):
    healthchecks.settings = MockSettings(HEALTHCHECKS={'API_KEY': 'this is a key'}, DEBUG=False, HOSTNAME='cronitor.io')
    healthchecks.Client.enqueue(healthcheck_instance)
    healthchecks.Client._flush_messages_to_log = lambda: ''
    healthchecks.Client.put()
    assert mock_put.call_count == 1, "requests.put not called once"
    assert 'Request failure' in healthchecks.Client._messages[0][1], \
        "Expected an error with 'Request failure', got '{}'".format(healthchecks.Client._messages[0][1])


def test_drain_queue_drains_the_queue(healthcheck_instance):
    healthchecks.settings = MockSettings(HEALTHCHECKS={'API_KEY': 'this is a key'}, DEBUG=True, HOSTNAME='cronitor.io')
    healthchecks.Client.enqueue(healthcheck_instance)

    assert len(healthchecks.Client.drain()) > 0, "Expected enqueued healthcheck from drain()"
    assert len(healthchecks.Client.drain()) == 0, "Expected empty drain() on second attempt"





# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from future.utils import python_2_unicode_compatible
from future.standard_library import install_aliases
install_aliases()
from django.conf.urls import url as django_url
from django.core.urlresolvers import reverse
from django.conf import settings
from urllib.parse import urlencode
import base64
import django.urls.exceptions
import hashlib
import json
import logging
import requests

ENDPOINT_URL = 'https://cronitor.io/v3/monitors'
DOCS_URL = 'https://cronitor.io/docs/django-healthchecks'

DEFAULTS = {
    'API_KEY': None,
    'HTTPS': False,
    'TAGS': [],
}


class HealthcheckError(RuntimeError):
    pass


@python_2_unicode_compatible
class Healthcheck(object):

    def __init__(self, route=None, args=None, kwargs=None, current_app=None, name=None, code=None, method='GET',
                 querystring=None, body=None, headers=None, cookies=None, assertions=None, tags=None, note=None,
                 interval_seconds=None, timeout_seconds=None):
        """ Define a healthcheck using a django url route that will be put to the Cronitor API

                route (str): Name of a url route. If omitted, must be set before resolve().

                args (list|tuple): Optional args to be passed to `reverse()` this route. Cannot be used with `kwargs`.

                kwargs (dict): Optional kwargs to be passed to `reverse()` this route. Cannot be used with `args`.

                current_app (str): If the app is namespaced or route name is not unique the `current_app` argument is
                                   needed for `reverse()`.

                name (str): Optional name for this monitor. If none is provided, a name will be generated.

                code (str): Optional monitor code. Use this to tie to an existing healthcheck on your Cronitor dashboard

                method (str): Request method used when performing this healthcheck.

                querystring (dict): Optional querystring parameters that will be appended to the URL

                body (str): Request body sent when performing this healthcheck if method is PUT, POST or PATCH.

                headers (dict): Optional request headers that will be sent when performing this healthcheck.

                cookies (dict): Optional cookies that will be sent when performing this healthcheck.

                assertions (dict): Optional assertions for this monitor. See API docs for details.

                tags (list|tuple): Optional tags for this monitor.

                note (string): Optional note that will be attached to this monitor.

                interval_seconds (int): Optional interval between healthcheck tests. If omitted, a default will be used.

                timeout_seconds (int): Optional timeout for this request, maximum of 10 seconds.
        """
        self.route = route
        self.args = args if args else ()
        self.kwargs = kwargs if kwargs else {}
        self.current_app = current_app
        self.name = name
        self.code = code
        self.method = method.upper()
        self.querystring = querystring if querystring else {}
        self.body = body
        self.headers = headers
        self.cookies = cookies
        self.assertions = assertions
        self.tags = tags
        self.note = note
        self.interval_seconds = interval_seconds
        self.timeout_seconds = timeout_seconds

        # When in DEBUG mode, create monitors in Dev mode
        self.is_dev = settings.DEBUG

        # These will be defined later during resolve():
        self._url = None
        self._defaultName = None

    def __str__(self):
        return self.name()

    def name(self):
        """ Retrieve the effective name of this healthcheck. """
        return self.name if self.name else self._defaultName

    def resolve(self):
        """ Because the route cannot be reversed into a URL at the same time its defined, we delay route resolution
        until we are ready to submit the healthchecks to the API. """
        self._url = HealthcheckUrl(
            path=self._reverse(),
            querystring=self.querystring
        )
        self._defaultName = self._create_name()
        self.code = self.code if self.code else self._create_code()

    def serialize(self):
        """ Serialize current instance details into valid API payload
        :return: dict """

        assert self.method in ('GET', 'POST', 'PUT', 'HEAD', 'OPTIONS', 'PATCH'), \
            "Healthcheck request method must be GET, POST, PUT, HEAD, OPTIONS or PATCH"

        request = {
            'url': self._url.url,
            'method': self.method
        }
        if self.cookies:
            assert isinstance(self.cookies, dict), "Healthcheck request cookies must be a dict"
            request['cookies'] = self.cookies
        if self.headers:
            assert isinstance(self.headers, dict), "Healthcheck request headers must be a dict"
            request['headers'] = self.headers
        if self.timeout_seconds:
            assert isinstance(self.timeout_seconds, int), "Healthcheck request timeout_seconds must be an int"
            request['timeout_seconds'] = self.timeout_seconds
        if self.body:
            request['body'] = self.body

        definition = {
            'type': 'healthcheck',
            'code': self.code,
            'defaultName': self._defaultName,
            'request': request,
            'dev': self.is_dev
        }

        if self.name:
            definition['name'] = self.name

        if self.assertions:
            assert isinstance(self.assertions, (list, tuple, set)), \
                "Healthcheck assertions must be a list, tuple or set"
            definition['rules'] = self.assertions

        if self.interval_seconds:
            assert isinstance(self.interval_seconds, int), \
                "Healthcheck interval_seconds must be an int"
            definition['request_interval_seconds'] = self.interval_seconds

        if self.tags:
            assert isinstance(self.tags, (list, tuple, set)), \
                "Healthcheck tags must be in a list, tuple or set"
            definition['tags'] = set(self.tags + _get_setting('TAGS'))
        elif _get_setting('TAGS'):
            assert isinstance(_get_setting('TAGS'), (list, tuple, set)), \
                "settings.HEALTHCHECKS['TAGS'] must be a list, tuple or set"
            definition['tags'] = _get_setting('TAGS')

        if self.note:
            definition['note'] = self.note

        return definition

    def _reverse(self):
        # First, try the route name with a namespace
        reverse_kwargs = {}
        if self.kwargs:
            reverse_kwargs['kwargs'] = self.kwargs
        elif self.args:
            reverse_kwargs['args'] = self.args

        if self.current_app:
            reverse_kwargs['current_app'] = self.current_app

        try:
            return reverse(self.route, **reverse_kwargs)
        except django.urls.exceptions.NoReverseMatch:
            raise HealthcheckError(
                'Could not reverse route for {}. '
                'Provide a `current_app` hint in your healthcheck definition.'.format(self.route)
            )

    def _create_name(self):
        """ Create a default name e.g. GET www.example.com/login
        :return: str """
        return '{} {}'.format(
            self.method,
            self._url.display
        )

    def _create_code(self):
        """ Generate a unique identifier for this monitor that can be used to update the monitor even if the name
        is changed on the Cronitor dashboard. Include is_dev in the hash to differentiate between dev and prod
        versions of a monitor
        :return: str """
        env = 'dev' if self.is_dev else 'prod'
        signature = hashlib.sha1(env.encode() + self._defaultName.encode())
        hash = base64.b64encode(signature.digest())
        return str(hash[:12]).replace('+', '').replace('/', '')


class HealthcheckUrl(object):

    url = None
    """ :type unicode Fully qualified URL that will be used in healthcheck """

    display = None
    """ :type unicode Shorter, prettier version of URL for display """

    def __init__(self, path, querystring):
        scheme = 'https://' if _get_setting('HTTPS') else 'http://'
        hostname = self._get_hostname()
        querystring = '?{}'.format(urlencode(querystring)) if querystring else ''
        self.url = '{}{}{}{}'.format(scheme, hostname, path, querystring)
        self.display = '{}{}'.format(hostname, path)

    def _get_hostname(self):
        """ Try to determine the hostname to use when making the healthcheck request
        :return: string
        :raises HealthcheckError """

        # First look in settings.HEALTHCHECKS['HOSTNAME']
        try:
            if _get_setting('HOSTNAME'):
                return _get_setting('HOSTNAME')
        except HealthcheckError:
            pass

        # Then try settings.HOSTNAME, if it exists
        try:
            if hasattr(settings, 'HOSTNAME') and len(settings.HOSTNAME):
                return settings.HOSTNAME
        except (AttributeError, TypeError):
            pass

        # Finally, pop the first value from settings.ALLOWED_HOSTS
        try:
            if hasattr(settings, 'ALLOWED_HOSTS') and len(settings.ALLOWED_HOSTS):
                return settings.ALLOWED_HOSTS[0]
        except (AttributeError, TypeError, KeyError):
            pass

        raise HealthcheckError(
            'Error: Could not determine hostname from settings.HEALTHCHECKS["HOSTNAME"], '
            'settings.HOSTNAME or settings.ALLOWED_HOSTS'
        )


class IdempotentHealthcheckClient(object):
    """
    Put enqueued healthchecks to the Cronitor API.
    """
    _queue = None
    """ We cannot `reverse()` a route at the same time the `url()` method is called. Queue healthchecks for later.
    :type list """

    _messages = None
    """ Messages written to a `django_auto_healthchecks.healthchecks` logger
    :type list """

    def __init__(self):
        self._queue = []
        self._messages = []

    def enqueue(self, healthcheck):
        """ Add a healthcheck instance to a queue for later processing.
        :param Healthcheck healthcheck:
        """
        self._queue.append(healthcheck)

    def drain(self):
        """ Drain enqueued healthchecks and return a list of Healthcheck objects
        :return: List[Healthcheck]"""
        healthchecks = {}
        for healthcheck in self._queue:
            # Routes cannot be reversed at the same time the healthcheck is defined, prepare them for use now.
            healthcheck.resolve()

            if healthcheck.code in healthchecks:
                self._messages.append((logging.WARN, 'Duplicate definition definition for {}, last one wins'.format(
                    healthcheck.name()
                )))

            healthchecks[healthcheck.code] = healthcheck

        self._queue = []
        return healthchecks.values()

    def put(self, additional_healthchecks=None):

        # If healthchecks have been defined in a batch and passed here, add them to the queue containing any
        # checks defined in urls.py file(s)
        map(self.enqueue, (additional_healthchecks or ()))
        healthchecks = self.drain()

        if len(healthchecks) == 0:
            self._messages.append(
                (logging.WARN, 'No healthchecks defined. See {} to get started.'.format(DOCS_URL))
            )
        else:
            try:
                payload = []
                for healthcheck in healthchecks:
                    try:
                        payload.append(healthcheck.serialize())
                    except AssertionError as e:
                        self._messages.append((
                            logging.ERROR,
                            'Healthcheck will not be published. Validation error: {}'.format(e)
                        ))

                api_key = _get_setting('API_KEY')
                if api_key:
                    try:
                        r = requests.put(ENDPOINT_URL, json=payload, auth=(api_key, ''), timeout=5)
                        if r.status_code != requests.codes.ok:
                            raise HealthcheckError(r.text)
                    except Exception as e:
                        self._messages.append((
                            logging.ERROR,
                            'Cronitor healthchecks could not be published. Request failure. Details:\n\n{}'.format(e)
                        ))
                else:
                    self._messages.append((
                        logging.ERROR,
                        'Missing Cronitor API key. Set settings.HEALTHCHECKS["API_KEY"] to publish healthchecks.'
                    ))

                if settings.DEBUG:
                    self._messages.append((
                        logging.INFO,
                        'DEV MODE: settings.DEBUG is True. Monitors will be created in Dev mode.'
                    ))

                self._messages.append((
                    logging.DEBUG,
                    'PUT {}:\n{}\n\n'.format(ENDPOINT_URL, json.dumps(payload, indent=2))
                ))

            except HealthcheckError as e:
                self._messages.append((logging.ERROR, str(e)))

        self._flush_messages_to_log()

    def _flush_messages_to_log(self):
        """ Write messages to the `django_auto_healthchecks.healthchecks` logger """
        logging.basicConfig()
        logger = logging.getLogger(__name__)
        [logger.log(msg[0], msg[1]) for msg in self._messages]
        self._messages = []


def url(regex, view, healthcheck=None, **kwargs):
    """ Drop-in replacement for django.conf.urls.url to create and update a Cronitor healthcheck when your app restarts.
    See https://cronitor.io/docs/healthchecks for details.
    :param regex:
    :param view:
    :param healthcheck: Use True for a simple GET healthcheck. Pass a dict to define custom assertions and notifications
    :param kwargs:
    :return:
    """

    if isinstance(healthcheck, Healthcheck):
        if isinstance(view, (list, tuple)):
            if settings.DEBUG:
                raise HealthcheckError('Healthchecks must be defined on individual routes')
        else:
            healthcheck.route = kwargs.get('name')
            Client.enqueue(healthcheck)

    return django_url(regex, view, **kwargs)


def put(healthchecks=()):
    """ Atomically create/update healthchecks with supplied list of Healthcheck instances. Use this to put healthchecks
    from your deploy script, or add healthchecks for third-party apps without having to modify their code.
    :param healthchecks: list[Healthcheck] of Healthcheck objects These healthchecks will be added to any defined in
           urls.py file(s). See https://cronitor.io/docs/django-healthchecks for details. """
    Client.put(healthchecks)


def _get_setting(key):
    """ For any given setting, look in the HEALTHCHECKS key of the django settings object and global key in settings obj.
    If it's not there, look for default in DEFAULTS
    :param key: Name of setting
    :return: * """
    if hasattr(settings, 'HEALTHCHECKS') and key in settings.HEALTHCHECKS:
        return settings.HEALTHCHECKS[key]

    if key in DEFAULTS:
        return DEFAULTS[key]

    raise HealthcheckError('Error: Could not find setting key {}'.format(key))


Client = IdempotentHealthcheckClient()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for `django_auto_healthchecks.healthchecks.Healthcheck` class.
"""

try:
    import mock
except ImportError:
    from unittest import mock

import pytest
import django_auto_healthchecks.healthchecks as healthchecks
from . import MockSettings


@pytest.fixture
def healthcheck_instance():
    def mock_resolve(self):
        self._url = healthchecks.HealthcheckUrl(
            path='/path/to',
            querystring={}
        )
        self._defaultName = self._create_name()
        self.code = self.code if self.code else self._create_code()

    healthchecks.settings = MockSettings(HEALTHCHECKS={'API_KEY': 'this is a key'}, DEBUG=True)
    instance = healthchecks.Healthcheck()
    instance.resolve = lambda: mock_resolve(instance)
    return instance


@mock.patch('django_auto_healthchecks.healthchecks.reverse', return_value='/path/to/endpoint')
def test_dev_mode_when_debug_true(mock_reverse):
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': 'cronitor.io'}, DEBUG=True)
    healthcheck = healthchecks.Healthcheck()
    healthcheck.resolve()
    payload = healthcheck.serialize()
    assert mock_reverse.call_count == 1, "Expected reverse() call once"
    assert 'dev' in payload, "Serialized payload missing 'dev' field"
    assert payload['dev'], "Expected dev to be True"

@mock.patch('django_auto_healthchecks.healthchecks.reverse', return_value='/path/to/endpoint')
def test_dev_mode_false_when_debug_false(mock_reverse):
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': 'cronitor.io'}, DEBUG=False)
    healthcheck = healthchecks.Healthcheck()
    healthcheck.resolve()
    payload = healthcheck.serialize()
    assert mock_reverse.call_count == 1, "Expected reverse() call once"
    assert 'dev' in payload, "Serialized payload missing 'dev' field"
    assert not payload['dev'], "Expected dev to be False"

@mock.patch('django_auto_healthchecks.healthchecks.reverse', return_value='/path/to/endpoint')
def test_dev_mode_changes_generated_code(mock_reverse):
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': 'cronitor.io'}, DEBUG=True)
    healthcheck = healthchecks.Healthcheck()
    healthcheck.resolve()
    devCode = healthcheck.code

    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': 'cronitor.io'}, DEBUG=False)
    healthcheck = healthchecks.Healthcheck()
    healthcheck.resolve()
    prodCode = healthcheck.code

    assert devCode != prodCode, "Expected DEBUG flag to change generated job code"

@mock.patch('django_auto_healthchecks.healthchecks.reverse', return_value='/path/to/endpoint')
def test_name_change_does_not_change_generated_code(mock_reverse):
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': 'cronitor.io'}, DEBUG=True)
    healthcheck = healthchecks.Healthcheck()
    healthcheck.resolve()
    devCode = healthcheck.code
    healthcheck = healthchecks.Healthcheck(name="Something better than the default name")
    healthcheck.resolve()
    prodCode = healthcheck.code

    assert devCode == prodCode, "Name change should not affect produced job code"


@mock.patch('django_auto_healthchecks.healthchecks.reverse', return_value='/path/to/endpoint')
def test_url_resolve_call(mock_reverse):
    route_name = 'example-route-name'
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': 'cronitor.io'}, DEBUG=True)
    healthcheck = healthchecks.Healthcheck(route=route_name)
    healthcheck.resolve()
    payload = healthcheck.serialize()
    assert mock_reverse.call_args == ((route_name,),), "Args not passed to mock django_url"
    assert '/path/to/endpoint' in payload['request']['url'], "Expected reverse() return path in generated request URL"


@mock.patch('django_auto_healthchecks.healthchecks.reverse', return_value='/path/to/endpoint')
def test_url_resolve_call_includes_current_app_when_provided(mock_reverse):
    route_name = 'example-route-name'
    current_app = 'example-app-name'
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': 'cronitor.io'}, DEBUG=True)
    healthcheck = healthchecks.Healthcheck(route=route_name, current_app=current_app)
    healthcheck.resolve()

    expected_args = ((route_name,), {'current_app': 'example-app-name'})
    assert mock_reverse.call_args == expected_args, "Args not passed to mock django_url"


@mock.patch('django_auto_healthchecks.healthchecks.reverse', return_value='/path/to/endpoint')
def test_monitor_tags_merged_with_settings_tags(mock_reverse):
    first_tags = ['Django', 'ExampleApp']
    second_tags = ['LandingPages', 'Django']
    merged_tags = ['Django', 'ExampleApp', 'LandingPages']

    healthchecks.settings = MockSettings(
        HEALTHCHECKS={'HOSTNAME': 'cronitor.io', 'TAGS': first_tags},
        DEBUG=False
    )
    healthcheck = healthchecks.Healthcheck(tags=second_tags)
    healthcheck.resolve()
    payload = healthcheck.serialize()
    assert set(payload['tags']) == set(merged_tags), "Tags not merged as expected"


@mock.patch('django_auto_healthchecks.healthchecks.reverse', return_value='/path/to/endpoint')
def test_serialized_payload_includes_default_parts(mock_reverse):
    healthchecks.settings = MockSettings(
        HEALTHCHECKS={'HOSTNAME': 'cronitor.io'},
        DEBUG=False
    )
    healthcheck = healthchecks.Healthcheck()
    healthcheck.resolve()
    payload_keys = healthcheck.serialize().keys()
    assert 'dev' in payload_keys, "Request payload should have 'dev' field"
    assert 'request' in payload_keys, "Request payload should have 'request' field"
    assert 'code' in payload_keys, "Request payload should have 'code' field"
    assert 'defaultName' in payload_keys, "Request payload should have 'defaultName' field"
    assert 'type' in payload_keys, "Request payload should have 'type' field"


@mock.patch('django_auto_healthchecks.healthchecks.reverse', return_value='/path/to/endpoint')
def test_serialized_payload_includes_optional_parts(mock_reverse):
    healthchecks.settings = MockSettings(
        HEALTHCHECKS={'HOSTNAME': 'cronitor.io'},
        DEBUG=False
    )
    healthcheck = healthchecks.Healthcheck(name='Example Name', tags=['tag1'], note='Example Note', assertions=[{
        'rule_type': 'response_body',
        'operator': 'contains',
        'value': 'Cosmo',
    }])
    healthcheck.resolve()
    payload_keys = healthcheck.serialize().keys()
    assert 'rules' in payload_keys, "Request payload should have 'rules' field"
    assert 'tags' in payload_keys, "Request payload should have 'tags' field"
    assert 'note' in payload_keys, "Request payload should have 'note' field"
    assert 'name' in payload_keys, "Request payload should have 'name' field"



# -*- coding: utf-8 -*-
from . import healthchecks

__version__ = '0.1.2'

url = healthchecks.url
""" url is a drop-in replacement for django URL that adds a new healthcheck kwarg """

put = healthchecks.put
""" Optionally, use the put method to batch create/update healthcheck definitions """

HealthcheckError = healthchecks.HealthcheckError
""" :type healthchecks.HealthcheckError Healthcheck definition exception """

Healthcheck = healthchecks.Healthcheck
""" :type healthchecks.Healthcheck """

Client = healthchecks.Client

default_app_config = 'django_auto_healthchecks.apps.HealthchecksAppConfig'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for `django_auto_healthchecks.healthchecks.HealthcheckUrl` class.
"""
from __future__ import unicode_literals

try:
    import mock
except ImportError:
    from unittest import mock

import django_auto_healthchecks.healthchecks as healthchecks
from . import MockSettings


def test_hostname_from_healthcheck_settings():
    test_hostname = 'cronitor.io'
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': test_hostname})
    HcUrl = healthchecks.HealthcheckUrl('/path/to/endpoint', None)
    assert test_hostname in HcUrl.url, "URL does not contain expected hostname"


def test_hostname_detection_fallback_to_settings_hostname():
    test_hostname = 'cronitor.io'
    healthchecks.settings = MockSettings(HOSTNAME=test_hostname)
    HcUrl = healthchecks.HealthcheckUrl('/path/to/endpoint', None)
    assert test_hostname in HcUrl.url, "URL does not contain expected hostname"


def test_hostname_detection_fallback_to_allowed_hosts():
    test_hostname = 'cronitor.io'
    healthchecks.settings = MockSettings(ALLOWED_HOSTS=(test_hostname,))
    HcUrl = healthchecks.HealthcheckUrl('/path/to/endpoint', None)
    assert test_hostname in HcUrl.url, "URL does not contain expected hostname"


def test_raise_exception_when_hostname_cannot_be_detected():
    healthchecks.settings = MockSettings()
    raised = False
    try:
        HcUrl = healthchecks.HealthcheckUrl('/path/to/endpoint', None)
    except healthchecks.HealthcheckError:
        raised = True
    finally:
        assert raised, "Expected HealthcheckError exception not raised"


def test_url_contains_all_expected_parts():
    test_hostname = 'cronitor.io'
    test_path = '/path/to/endpoint'
    test_querystring = {
        'foo': 'bar',
        'bar': 'foo'
    }
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': test_hostname, 'HTTPS': False})
    HcUrl = healthchecks.HealthcheckUrl(test_path, test_querystring)
    assert 'foo=bar' in HcUrl.url, "URL does not expected contain querystring part"
    assert 'bar=foo' in HcUrl.url, "URL does not expected contain querystring part"
    assert 'http' in HcUrl.url, "URL does not contain scheme"
    assert test_hostname in HcUrl.url, "URL does not contain hostname"
    assert test_path in HcUrl.url, "URL does not contain path"


def test_https_enabled_healthcheck_settings():
    test_hostname = 'cronitor.io'
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': test_hostname, 'HTTPS': True})
    HcUrl = healthchecks.HealthcheckUrl('/path/to/endpoint', None)
    assert test_hostname in HcUrl.url, "URL does not contain expected hostname"
    assert 'https://' in HcUrl.url, "URL does not contain https://"


def test_https_disabled_healthcheck_settings():
    test_hostname = 'cronitor.io'
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': test_hostname, 'HTTPS': False})
    HcUrl = healthchecks.HealthcheckUrl('/path/to/endpoint', None)
    assert test_hostname in HcUrl.url, "URL does not contain expected hostname"
    assert 'http://' in HcUrl.url, "URL does not contains http://"


def test_https_disabled_by_default():
    test_hostname = 'cronitor.io'
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': test_hostname})
    HcUrl = healthchecks.HealthcheckUrl('/path/to/endpoint', None)
    assert 'http://' in HcUrl.url, "URL does not contains http://"


def test_querystring_in_url_not_display_url():
    test_hostname = 'cronitor.io'
    test_path = '/path/to/endpoint'
    test_querystring = {
        'foo': 'bar',
        'bar': 'foo'
    }
    healthchecks.settings = MockSettings(HEALTHCHECKS={'HOSTNAME': test_hostname, 'HTTPS': False})
    HcUrl = healthchecks.HealthcheckUrl(test_path, test_querystring)
    assert 'foo=bar' in HcUrl.url, "URL does not contains querystring"
    assert 'bar=foo' in HcUrl.url, "URL does not contains querystring"
    assert 'foo=bar' not in HcUrl.display, "Display URL unexpectedly containss querystring"
    assert 'bar=foo' not in HcUrl.display, "Display URL unexpectedly containss querystring"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tests for `django_auto_healthchecks.healthchecks.url` function, a proxy for django `django.conf.urls.url`.
"""

try:
    import mock
except ImportError:
    from unittest import mock

import django_auto_healthchecks.healthchecks as healthchecks
from . import MockSettings


@mock.patch('django_auto_healthchecks.healthchecks.django_url')
def test_url_calls_django_url(mock_url):
    url_args = ('regex', 'view')
    healthchecks.url(*url_args)
    assert mock_url.call_count == 1, "Mock django_url not called once"
    assert mock_url.call_args == (url_args,), "Args not passed to mock django_url"


@mock.patch('django_auto_healthchecks.healthchecks.Client.enqueue')
@mock.patch('django_auto_healthchecks.healthchecks.django_url')
def test_url_enqueues_healthcheck(mock_url, mock_enqueue):
    healthchecks.settings = MockSettings(DEBUG=True)
    url_args = ('regex', 'view', healthchecks.Healthcheck())
    healthchecks.url(*url_args)
    assert mock_enqueue.call_count == 1, "Mock Client.enqueue not called once"
    assert mock_url.call_count == 1, "Mock django_url not called once"


@mock.patch('django_auto_healthchecks.healthchecks.Client.enqueue')
@mock.patch('django_auto_healthchecks.healthchecks.django_url')
def test_url_strips_healthcheck_before_calling_django_url(mock_url, mock_enqueue):
    healthchecks.settings = MockSettings(DEBUG=True)
    url_args = ('regex', 'view', healthchecks.Healthcheck())
    expected_django_args = ('regex', 'view')
    healthchecks.url(*url_args)
    assert mock_enqueue.call_count == 1, "Mock Client.enqueue not called once"
    assert mock_url.call_count == 1, "Mock django_url not called once"
    assert mock_url.call_args == (expected_django_args,), "Expected args not passed to mock django_url"


def test_url_with_include_raises_exception_when_in_debug_mode():
    healthchecks.settings = MockSettings(DEBUG=True)
    raised = False
    try:
        healthchecks.url('regex', ('included', 'url', 'tuple'), healthchecks.Healthcheck())
    except healthchecks.HealthcheckError:
        raised = True
    finally:
        assert raised, "Expected HealthcheckError exception not raised"


def test_url_with_include_fails_silently_when_not_in_debug_mode():
    healthchecks.settings = MockSettings(DEBUG=False)
    raised = False
    try:
        healthchecks.url('regex', ('included', 'url', 'tuple'), healthchecks.Healthcheck())
    except healthchecks.HealthcheckError:
        raised = True
    finally:
        assert not raised, "Unexpected HealthcheckError exception raised"


@mock.patch('django_auto_healthchecks.healthchecks.Client.enqueue')
@mock.patch('django_auto_healthchecks.healthchecks.django_url')
def test_url_without_healthcheck_param(mock_url, mock_enqueue):
    healthchecks.settings = MockSettings(DEBUG=True)
    url_args = ('regex', 'view')
    healthchecks.url(*url_args)
    assert mock_enqueue.call_count == 0, "Unexpected call to mock Client.enqueue"
    assert mock_url.call_count == 1, "Mock django_url not called once"
    assert mock_url.call_args == (url_args,), "Expected args not passed to mock django_url"


@mock.patch('django_auto_healthchecks.healthchecks.Client.enqueue')
@mock.patch('django_auto_healthchecks.healthchecks.django_url')
def test_route_name_added_to_healthcheck(mock_url, mock_enqueue):
    healthchecks.settings = MockSettings(DEBUG=True)
    route_name = 'test-route'
    healthchecks.url('regex', 'view', healthchecks.Healthcheck(), name=route_name)
    assert mock_enqueue.call_count == 1, "Mock Client.enqueue not called once"
    assert mock_enqueue.call_args[0][0].route == route_name, "Mock Client.enqueue not called once"



# -*- coding: utf-8 -*-
from django.apps import AppConfig
from django.core.urlresolvers import reverse
import django.urls.exceptions
from . import healthchecks


class HealthchecksAppConfig(AppConfig):
    """ Attach into the Django app startup.
    At this point the URL resolver cache has not been warmed yet. """
    name = 'django_auto_healthchecks'

    def ready(self):
        try:
            reverse('request-a-route-to-parse-urls-and-populate-healthchecks')
        except django.urls.exceptions.NoReverseMatch:
            pass
        finally:
            # Any healthecks defined in urls.py will be parsed and ready to PUT now.
            healthchecks.put()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools.command.test import test as TestCommand
from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    "requests==2.13.0",
    "future==0.16.0"
]

test_requirements = requirements + [
    "pytest",
    "django==1.10.1",
    "mock==2.0.0"
]


class PyTest(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        pytest.main(self.test_args)

setup(
    name='django_auto_healthchecks',
    version='0.1.2',
    description="Create application health checks directly from your django urls.py",
    long_description=readme + '\n\n' + history,
    author="Cronitor.io",
    author_email='support@cronitor.io',
    url='https://github.com/cronitorio/django_auto_healthchecks',
    packages=[
        'django_auto_healthchecks',
    ],
    package_dir={'django_auto_healthchecks':
                 'django_auto_healthchecks'},
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='django_auto_healthchecks',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests.test_healthchecks',
    tests_require=test_requirements,
    cmdclass={'test': PyTest},
)
